// This file is part of Lx.
//
// Copyright 2003, 2004, Norman Gray (norman@astro.gla.ac.uk).
// Released under the terms of the GNU General Public Licence:
// see the file LICENCE for details.


package uk.me.nxg.lx;

import java.util.List;

/**
 * An implementation of the SAX2 ContentHandler interface which simply
 * traces all calls, and appends information about them to a List.
 * For debugging.
 */
public class TraceContentHandler
        extends org.xml.sax.helpers.DefaultHandler 
        implements org.xml.sax.DocumentHandler {

    private List eventlist;
    private boolean showQN;
    private boolean debug = false;

    public TraceContentHandler(List eventlist) {
        this.eventlist = eventlist;
        showQN = true;
    }
	
    public void showQNames(boolean value) {
        showQN = value;
    }

    public void debug(boolean value) {
        debug = value;
    }
        
    public void startDocument() {
        if (debug)
            System.err.println("startDocument");
        eventlist.add("startDocument");
    }

    /* SAX2 ContentHandler.startElement */
    public void startElement(String uri, String localName, String qName,
                             org.xml.sax.Attributes atts) {
        StringBuffer sb = new StringBuffer("startElement {");
        sb.append(uri).append("}");
        sb.append(localName);
        if (showQN)
            sb.append('/').append(qName);
        sb.append("[");
        if (atts.getLength() > 0) {
            // For repeatability, list the attribute-value strings in 
            // alphabetical order
            java.util.SortedSet ss = new java.util.TreeSet();
            for (int i=0; i<atts.getLength(); i++) {
                StringBuffer sbi = new StringBuffer();
                String ns = atts.getURI(i);

                if (ns == null) {
                    // an error!
                    sbi.append("NULL-NS");

                } else {
                
                    if (atts.getQName(i).startsWith("xml")) {
                        sbi.append("NS:").append(atts.getQName(i));
                    } else if (ns.length() > 0) {
                        sbi.append('{').append(ns).append("}")
                                .append(atts.getLocalName(i));
                        if (showQN)
                            sbi.append("/").append(atts.getQName(i));
                    } else {
                        sbi.append(atts.getLocalName(i));
                    }
                }
                
                sbi.append('=').append(atts.getValue(i));
                ss.add(sbi.toString());
            }
            boolean first = true;
            for (java.util.Iterator si = ss.iterator(); si.hasNext(); ) {
                if (first)
                    first = false;
                else
                    sb.append(",");
                sb.append((String)si.next());
            }
        }
        sb.append("]");
        String s = sb.toString();
        if (debug)
            System.err.println(s);
        eventlist.add(s);
    }

    /* SAX1 DocumentHandler.startElement */
    public void startElement(java.lang.String name,
                             org.xml.sax.AttributeList atts) {
        StringBuffer sb = new StringBuffer("startElement1 ");
        sb.append(name);
        sb.append('[');
        if (atts.getLength() > 0) {
            java.util.SortedSet ss = new java.util.TreeSet();
            for (int i=0; i<atts.getLength(); i++) {
                StringBuffer sbi = new StringBuffer();
                sbi.append(atts.getName(i))
                        .append('=')
                        .append(atts.getValue(i));
                ss.add(sbi.toString());
            }
            boolean first = true;
            for (java.util.Iterator si = ss.iterator(); si.hasNext(); ) {
                if (first)
                    first = false;
                else
                    sb.append(',');
                sb.append((String)si.next());
            }
        }
        sb.append(']');
        String s = sb.toString();
        if (debug)
            System.err.println(s);
        eventlist.add(s);
    }

    public void startPrefixMapping(String prefix, String uri) {
        StringBuffer sb = new StringBuffer("map+ ");
        sb.append(prefix).append(">").append(uri);
        String s = sb.toString();
        if (debug)
            System.err.println(s);
        eventlist.add(s);
    }

    public void endPrefixMapping(String prefix) {
        if (debug)
            System.err.println("map- " + prefix);
        eventlist.add("map- " + prefix);
    }

    /* SAX2 endElement */
    public void endElement(String uri, String localName, String qName) {
        StringBuffer sb = new StringBuffer("endElement {");
        sb.append(uri).append("}");
        sb.append(localName);
        if (showQN) 
            sb.append('/').append(qName);
        String s = sb.toString();
        if (debug)
            System.err.println(s);
        eventlist.add(s);
    }

    /* SAX1 endElement */
    public void endElement(String name) {
        StringBuffer sb = new StringBuffer("endElement1 ");
        sb.append(name);
        String s = sb.toString();
        if (debug)
            System.err.println(s);
        eventlist.add(s);
    }

    public void characters(char[] ch, int start, int length) {
        StringBuffer sb = new StringBuffer("characters ");
        sb.append(length).append(':');
        if (start == 0 && length == ch.length)
            sb.append(ch);
        else
            for (int i=start; i<start+length; i++)
                sb.append(ch[i]);
        String s = sb.toString();
        if (debug)
            System.err.println(s);
        eventlist.add(s);
    }

    public void endDocument() {
        if (debug)
            System.err.println("endDocument");
        eventlist.add("endDocument");
    }
}
