// This file is part of Lx.
//
// Copyright 2003, 2004, Norman Gray (norman@astro.gla.ac.uk).
// Released under the terms of the GNU General Public Licence:
// see the file LICENCE for details.


package uk.me.nxg.lx.helpers;

import junit.framework.Assert;
import junit.framework.TestCase;

import uk.me.nxg.lx.TraceContentHandler;

import java.util.List;
import java.io.StringReader;

import org.xml.sax.XMLReader;
import org.xml.sax.Parser;
import org.xml.sax.InputSource;


public class MultiParserTest
        extends TestCase {

    public MultiParserTest(String name) {
        super(name);
    }

    public void testParsing()
            throws Exception {

        String testlx = "   (gi1 a1: v1 text)  ";
        String testxml = "   <gi1 a1='v1'  >text</gi1> ";
        String[] expectedStrings1 = {
            "startDocument",
            "startElement1 gi1[a1=v1]",
            "characters 4:text",
            "endElement1 gi1",
            "endDocument",
        };
        String[] expectedStrings2 = {
            "startDocument",
            "startElement {}gi1[a1=v1]",
            "characters 4:text",
            "endElement {}gi1",
            "endDocument",
        };

        List eventlist = new java.util.LinkedList();
        TraceContentHandler tch = new TraceContentHandler(eventlist);
        tch.showQNames(false);

        Parser p = new MultiParser();
        p.setDocumentHandler(tch);
        p.parse(new InputSource(new StringReader(testlx)));
        assertListEquals(expectedStrings1, eventlist);

        XMLReader r = new MultiParser();
        r.setContentHandler(tch);
        r.parse(new InputSource(new StringReader(testlx)));
        assertListEquals(expectedStrings2, eventlist);

        p = new MultiParser();
        p.setDocumentHandler(tch);
        p.parse(new InputSource(new StringReader(testxml)));
        assertListEquals(expectedStrings1, eventlist);

        r = new MultiParser();
        r.setContentHandler(tch);
        r.parse(new InputSource(new StringReader(testxml)));
        assertListEquals(expectedStrings2, eventlist);
    }

    /* Same as above, but with namespacing */
    public void testParsingNS()
            throws Exception {

        String testlx
                = "   (lx:namespace ((a \"urn:1\")) (a:gi1 a1: v1 text))";
        String testxml
                = "   <a:gi1 xmlns:a=\"urn:1\" a1='v1'  >text</a:gi1> ";
        String[] expectedStrings1 = {
            "startDocument",
            "startElement1 a:gi1[a1=v1,xmlns:a=urn:1]",
            "characters 4:text",
            "endElement1 a:gi1",
            "endDocument",
        };
        String[] expectedStrings2 = {
            "startDocument",
            "map+ a>urn:1",
            "startElement {urn:1}gi1[a1=v1]",
            "characters 4:text",
            "endElement {urn:1}gi1",
            "map- a",
            "endDocument",
        };

        List eventlist = new java.util.LinkedList();
        TraceContentHandler tch = new TraceContentHandler(eventlist);
        tch.showQNames(false);

        Parser p = new MultiParser();
        p.setDocumentHandler(tch);
        p.parse(new InputSource(new StringReader(testlx)));
        assertListEquals(expectedStrings1, eventlist);

        XMLReader r = new MultiParser();
        r.setContentHandler(tch);
        r.parse(new InputSource(new StringReader(testlx)));
        assertListEquals(expectedStrings2, eventlist);

        p = new MultiParser();
        p.setDocumentHandler(tch);
        p.parse(new InputSource(new StringReader(testxml)));
        assertListEquals(expectedStrings1, eventlist);

        r = new MultiParser();
        r.setContentHandler(tch);
        r.parse(new InputSource(new StringReader(testxml)));
        assertListEquals(expectedStrings2, eventlist);
    }

    /* ******************** Helper classes ******************** */

    private void assertListEquals(String[] expected, List l) {
        for (int i=0; i<expected.length; i++) {
            String head = (String)l.remove(0);
            assertNotNull(head);
            assertEquals(expected[i], head);
        }
        assertEquals(0, l.size());
    }
}
