// This file is part of Lx.
//
// Copyright 2003, 2004, Norman Gray (norman@astro.gla.ac.uk).
// Released under the terms of the GNU General Public Licence:
// see the file LICENCE for details.


package uk.me.nxg.lx.reader;

import uk.me.nxg.lx.LxException;

import junit.framework.Assert;
import junit.framework.TestCase;


public class AttributesTest 
	extends TestCase {

    public AttributesTest(String name) {
	super(name);
    }

    public void testAttributes()
	    throws Exception {

	Attributes atts = new Attributes();
        String ns = "http://example.org/ns";
        Stack s = new Stack();

        Stack.Mappings m = s.newMapping();
        m.mapPrefix("x", ns);
        m.mapPrefix("y", ns);
        s.push(m);

        atts.add(new QName("x:one", s), "first");
        atts.add(new QName("two", s), "second");
        atts.add(new QName("three", s), "");
        atts.add(new QName("y:four", s), "");

	assertEquals(4, atts.getLength());

	assertEquals(ns, atts.getURI(0));
	assertEquals("", atts.getURI(1));
	assertEquals("", atts.getURI(2));
	assertEquals(ns, atts.getURI(3));
	assertNull(atts.getURI(4));

	assertEquals("one", atts.getLocalName(0));
	assertEquals("two", atts.getLocalName(1));
	assertEquals("three", atts.getLocalName(2));
	assertEquals("four", atts.getLocalName(3));
	assertNull(atts.getLocalName(4));

	// Is the following a correct test?  What does `... or the empty
	// string if none is available' mean?
	assertEquals("x:one", atts.getQName(0));
	assertEquals("two", atts.getQName(1));
	assertEquals("three", atts.getQName(2));
	assertEquals("y:four", atts.getQName(3));
	assertNull(atts.getQName(4));

	for (int i=0; i<atts.getLength(); i++)
	    assertEquals("CDATA", atts.getType(i));
	assertNull(atts.getType(4));

	assertEquals("first", atts.getValue(0));
	assertEquals("second", atts.getValue(1));
	assertEquals("", atts.getValue(2));
	assertEquals("", atts.getValue(3));
	assertNull(atts.getValue(4));

	// Don't need to do following tests on empty-value atts
	// "three" and "four"

	assertEquals(0, atts.getIndex(ns, "one"));
	assertEquals(-1, atts.getIndex("http://another", "one"));
	assertEquals(-1, atts.getIndex(null, "one"));
	assertEquals(-1, atts.getIndex("", "one"));
	assertEquals(-1, atts.getIndex(ns, "two"));

	assertEquals(1, atts.getIndex("", "two"));
	assertEquals(-1, atts.getIndex(ns, "two"));
	assertEquals(-1, atts.getIndex("http://another", "two"));

	assertEquals(0, atts.getIndex("x:one"));
	assertEquals(-1, atts.getIndex("y:one"));
	assertEquals(-1, atts.getIndex("one"));
	assertEquals(1, atts.getIndex("two"));
	assertEquals(-1, atts.getIndex("x:two"));
	assertEquals(-1, atts.getIndex(""));
	assertEquals(-1, atts.getIndex(null));
	
	assertEquals("CDATA", atts.getType(ns, "one"));
	assertNull(atts.getType("http://another", "one"));
	assertNull(atts.getType(null, "one"));
	assertNull(atts.getType("", "one"));
	assertNull(atts.getType(ns, "two"));

	assertEquals("CDATA", atts.getType("", "two"));
	assertNull(atts.getType(ns, "two"));
	assertNull(atts.getType("http://another", "two"));

	assertEquals("CDATA", atts.getType("x:one"));
	assertNull(atts.getType("y:one"));
	assertNull(atts.getType("one"));
	assertEquals("CDATA", atts.getType("two"));
	assertNull(atts.getType("x:two"));
	assertNull(atts.getType(""));
	assertNull(atts.getType(null));
    }

    public void testNSAttributes()
            throws Exception {

        Attributes atts = new Attributes();
        String ns1 = "http://example.org/ns/one";
        String ns2 = "http://example.org/ns/two";
        Stack s = new Stack();

        atts.addNSAttribute(null, ns1);
        atts.addNSAttribute("p", ns2);

        // following should fail
        String[] bad = {
            "",                 // blank
            " ",                // space illegal
            "n s",              // ...
            "n:",               // colon
            "2n",               // bad start char
        };
        for (int i=0; i<bad.length; i++) {
            try {
                atts.addNSAttribute(bad[i], ns2);
                fail("Adding namespace prefix " + bad[i] + " did not fail");
            } catch (LxException e) {
                // OK
            }
        }

        assertEquals(2, atts.getLength());

        assertEquals("", atts.getURI(0));
        assertEquals("xmlns", atts.getLocalName(0));
        assertEquals("xmlns", atts.getQName(0));
        assertEquals(ns1, atts.getValue(0));

        assertEquals("", atts.getURI(1));
        assertEquals("xmlns:p", atts.getLocalName(1));
        assertEquals("xmlns:p", atts.getQName(1));
        assertEquals(ns2, atts.getValue(1));

        assertNull(atts.getLocalName(2));
        assertNull(atts.getQName(2));
        assertNull(atts.getValue(2));
    }
}
